package de.fhdw.gaming.memory;

import java.util.Map;

import de.fhdw.gaming.core.domain.Game;
import de.fhdw.gaming.core.domain.Move;

/**
 * A class managing all memories for all players.
 */
public interface Brain {
    
    /**
     * .
     * @return
     */
    int getLength();
    
    /**
     * Sets the amount of outcomes that the brain can remember. Can be used during a game.
     * @param length
     */
    void setMemoryLength(int length) throws IllegalArgumentException;

    /**
     * Adds the outcomes of the current game for each player to the memory. Each combination of active player,
     * their opponent, and the strategy of the active player requires a new {@link Key} for which the outcomes
     * will be saved. In a 2 player game without changing strategies there should
     * only ever be two {@link Key}s.
     * @param game
     */
    void rememberOutcome(Game<?, ?, ?, ?> game);
    
    /**
     * .
     * @param key
     * @param move
     */
    public void rememberMyMove(Key key, Move<?, ?> move);
    
    /**
     * .
     * @param key
     * @param move
     */
    public void rememberOpponentMove(Key key, Move<?, ?> move);

    /**
     * Returns the {@link ShiftList} containing the outcomes for a certain {@link Key}.
     * That means a certain combination of player the memory is for,
     * their opponent and the strategy used by the active player.
     * @param key {@link Key}
     */
    ShiftList getMemory(Key key);

    /**
     * Returns the entirety of the memory. It can be used to acquire any
     * {@link ShiftList} of outcomes with the right key.
     */
    Map<Key, ShiftList> getMemory();

}
